@@ -11,16 +11,18 @@
 
 #include <QSignalSpy>
 #include <QStandardPaths>
+#include <QTemporaryDir>
 #include <QTemporaryFile>
 #include <QTest>
 #include <kdesktopfile.h>
-#include <qtemporarydir.h>
 
 #include <kauthorized.h>
 #include <kconfiggroup.h>
 #include <kconfigwatcher.h>
 #include <ksharedconfig.h>
 
+#include <private/qlocale_p.h> // QSystemLocale
+
 #ifdef Q_OS_UNIX
 #include <utime.h>
 #endif
@@ -28,6 +30,8 @@
 #include <unistd.h> // getuid
 #endif
 
+using namespace Qt::StringLiterals;
+
 KCONFIGGROUP_DECLARE_ENUM_QOBJECT(KConfigTest, Testing)
 KCONFIGGROUP_DECLARE_FLAGS_QOBJECT(KConfigTest, Flags)
 
@@ -75,6 +79,7 @@ static const QDateTime s_date_time_tz = [] {
     dt.setTimeZone(QTimeZone("EST"));
     return dt;
 }();
+static const QTime s_time_entry(21, 42, 37);
 static const QStringList s_stringlist_entry{QStringLiteral("Hello,"), QStringLiteral("World")};
 static const QStringList s_stringlist_empty_entry{};
 static const QStringList s_stringlist_just_empty_element{QString{}};
@@ -170,6 +175,7 @@ void KConfigTest::initTestCase()
     cg.writeEntry("dateTimeWithMSEntry", s_date_time_with_ms_entry);
     cg.writeEntry("dateTimeUtc", s_date_time_utc);
     cg.writeEntry("dateTimeTz", s_date_time_tz);
+    cg.writeEntry("timeEntry", s_time_entry);
 
     KConfigGroup ct = cg;
     cg = KConfigGroup(&ct, QStringLiteral("Nested Group 1"));
@@ -486,7 +492,7 @@ void KConfigTest::testLists()
 
     QCOMPARE(sc3.readEntry("listOfIntsEntry1"), QString::fromLatin1("1,2,3,4"));
     QList<int> expectedIntList = s_int_listentry1;
-    QVERIFY(sc3.readEntry("listOfIntsEntry1", QList<int>()) == expectedIntList);
+    QCOMPARE(sc3.readEntry("listOfIntsEntry1", QList<int>()), expectedIntList);
 
     QCOMPARE(QVariant(sc3.readEntry("variantListEntry", s_variantlist_entry)).toStringList(), QVariant(s_variantlist_entry).toStringList());
 
@@ -612,6 +618,7 @@ void KConfigTest::testComplex()
     QCOMPARE(sc3.readEntry("dateTimeTz", QDateTime()).timeZone(), s_date_time_tz.timeZone());
     QCOMPARE(sc3.readEntry("dateTimeUtc", QDateTime()).timeRepresentation().id(), s_date_time_utc.timeRepresentation().id());
     QCOMPARE(sc3.readEntry("dateTimeEntry", QDate()), s_date_time_entry.date());
+    QCOMPARE(sc3.readEntry("timeEntry", QTime()), s_time_entry);
 }
 
 void KConfigTest::testEnums()
@@ -630,7 +637,7 @@ void KConfigTest::testEnums()
     QVERIFY(sc3.readEntry("enum-100", Ones) != Tens);
 
     QCOMPARE(sc3.readEntry("flags-bit0"), QStringLiteral("bit0"));
-    QVERIFY(sc3.readEntry("flags-bit0", Flags()) == bit0);
+    QCOMPARE(sc3.readEntry("flags-bit0", Flags()), bit0);
 
     int eid = staticMetaObject.indexOfEnumerator("Flags");
     QVERIFY(eid != -1);
@@ -638,7 +645,7 @@ void KConfigTest::testEnums()
     Flags bitfield = bit0 | bit1;
 
     QCOMPARE(sc3.readEntry("flags-bit0-bit1"), QString::fromLatin1(me.valueToKeys(bitfield)));
-    QVERIFY(sc3.readEntry("flags-bit0-bit1", Flags()) == bitfield);
+    QCOMPARE(sc3.readEntry("flags-bit0-bit1", Flags()), bitfield);
 }
 
 void KConfigTest::testEntryMap()
@@ -679,57 +686,63 @@ void KConfigTest::testInvalid()
     list << 1;
     sc3.writeEntry(QStringLiteral("badList"), list);
 
-    QVERIFY(sc3.readEntry("badList", QPoint()) == QPoint());
-    QVERIFY(sc3.readEntry("badList", QRect()) == QRect());
-    QVERIFY(sc3.readEntry("badList", QSize()) == QSize());
-    QVERIFY(sc3.readEntry("badList", QDate()) == QDate());
-    QVERIFY(sc3.readEntry("badList", QDateTime()) == QDateTime());
+    QCOMPARE(sc3.readEntry("badList", QPoint()), QPoint());
+    QCOMPARE(sc3.readEntry("badList", QRect()), QRect());
+    QCOMPARE(sc3.readEntry("badList", QSize()), QSize());
+    QCOMPARE(sc3.readEntry("badList", QDate()), QDate());
+    QCOMPARE(sc3.readEntry("badList", QTime()), QTime());
+    QCOMPARE(sc3.readEntry("badList", QDateTime()), QDateTime());
 
     // 2 element list
     list << 2;
     sc3.writeEntry("badList", list);
 
-    QVERIFY(sc3.readEntry("badList", QRect()) == QRect());
-    QVERIFY(sc3.readEntry("badList", QDate()) == QDate());
-    QVERIFY(sc3.readEntry("badList", QDateTime()) == QDateTime());
+    QCOMPARE(sc3.readEntry("badList", QRect()), QRect());
+    QCOMPARE(sc3.readEntry("badList", QDate()), QDate());
+    QCOMPARE(sc3.readEntry("badList", QTime()), QTime());
+    QCOMPARE(sc3.readEntry("badList", QDateTime()), QDateTime());
 
     // 3 element list
     list << 303;
     sc3.writeEntry("badList", list);
 
-    QVERIFY(sc3.readEntry("badList", QPoint()) == QPoint());
-    QVERIFY(sc3.readEntry("badList", QRect()) == QRect());
-    QVERIFY(sc3.readEntry("badList", QSize()) == QSize());
-    QVERIFY(sc3.readEntry("badList", QDate()) == QDate()); // out of bounds
-    QVERIFY(sc3.readEntry("badList", QDateTime()) == QDateTime());
+    QCOMPARE(sc3.readEntry("badList", QPoint()), QPoint());
+    QCOMPARE(sc3.readEntry("badList", QRect()), QRect());
+    QCOMPARE(sc3.readEntry("badList", QSize()), QSize());
+    QCOMPARE(sc3.readEntry("badList", QDate()), QDate()); // out of bounds
+    QCOMPARE(sc3.readEntry("badList", QTime()), QTime());
+    QCOMPARE(sc3.readEntry("badList", QDateTime()), QDateTime());
 
     // 4 element list
     list << 4;
     sc3.writeEntry("badList", list);
 
-    QVERIFY(sc3.readEntry("badList", QPoint()) == QPoint());
-    QVERIFY(sc3.readEntry("badList", QSize()) == QSize());
-    QVERIFY(sc3.readEntry("badList", QDate()) == QDate());
-    QVERIFY(sc3.readEntry("badList", QDateTime()) == QDateTime());
+    QCOMPARE(sc3.readEntry("badList", QPoint()), QPoint());
+    QCOMPARE(sc3.readEntry("badList", QSize()), QSize());
+    QCOMPARE(sc3.readEntry("badList", QDate()), QDate());
+    QCOMPARE(sc3.readEntry("badList", QTime()), QTime());
+    QCOMPARE(sc3.readEntry("badList", QDateTime()), QDateTime());
 
     // 5 element list
     list[2] = 3;
     list << 5;
     sc3.writeEntry("badList", list);
 
-    QVERIFY(sc3.readEntry("badList", QPoint()) == QPoint());
-    QVERIFY(sc3.readEntry("badList", QRect()) == QRect());
-    QVERIFY(sc3.readEntry("badList", QSize()) == QSize());
-    QVERIFY(sc3.readEntry("badList", QDate()) == QDate());
-    QVERIFY(sc3.readEntry("badList", QDateTime()) == QDateTime());
+    QCOMPARE(sc3.readEntry("badList", QPoint()), QPoint());
+    QCOMPARE(sc3.readEntry("badList", QRect()), QRect());
+    QCOMPARE(sc3.readEntry("badList", QSize()), QSize());
+    QCOMPARE(sc3.readEntry("badList", QDate()), QDate());
+    QCOMPARE(sc3.readEntry("badList", QTime()), QTime());
+    QCOMPARE(sc3.readEntry("badList", QDateTime()), QDateTime());
 
     // 6 element list
     list << 6;
     sc3.writeEntry("badList", list);
 
-    QVERIFY(sc3.readEntry("badList", QPoint()) == QPoint());
-    QVERIFY(sc3.readEntry("badList", QRect()) == QRect());
-    QVERIFY(sc3.readEntry("badList", QSize()) == QSize());
+    QCOMPARE(sc3.readEntry("badList", QPoint()), QPoint());
+    QCOMPARE(sc3.readEntry("badList", QRect()), QRect());
+    QCOMPARE(sc3.readEntry("badList", QSize()), QSize());
+    QCOMPARE(sc3.readEntry("badList", QTime()), QTime());
 }
 
 void KConfigTest::testChangeGroup()
@@ -762,12 +775,22 @@ void KConfigTest::testDeleteEntry()
     KConfig sc(configFile);
     KConfigGroup group(&sc, QStringLiteral("Hello"));
 
+    QVERIFY(!sc.isDirty());
     group.deleteEntry("DelKey");
+    QVERIFY(sc.isDirty());
     QCOMPARE(group.readEntry("DelKey", QStringLiteral("Fietsbel")), QStringLiteral("Fietsbel"));
 
     QVERIFY(group.sync());
     Q_ASSERT(!readLines(configFile).contains("DelKey=ToBeDeleted\n"));
     QCOMPARE(group.readEntry("DelKey", QStringLiteral("still deleted")), QStringLiteral("still deleted"));
+
+    // deleting a deleted key again should not mark it as dirty
+    group.deleteEntry("DelKey");
+    QVERIFY(!sc.isDirty());
+
+    // and neither should deleting an unknown key
+    group.deleteEntry("DoesNotExist");
+    QVERIFY(!sc.isDirty());
 }
 
 void KConfigTest::testDelete()
@@ -1311,7 +1334,7 @@ void KConfigTest::testReparent()
 
     QVERIFY(!parent.hasGroup(name));
 
-    QVERIFY(group.entryMap() == originalMap);
+    QCOMPARE(group.entryMap(), originalMap);
 
     group.reparent(&parent); // see if it can be made a sub-group of another group
     QVERIFY(parent.hasGroup(name));
@@ -1538,6 +1561,43 @@ void KConfigTest::testLocaleConfig()
     QFile::remove(file);
 }
 
+void KConfigTest::testLocaleConfigWithOverrideLanguage()
+{
+    // simulate usage of language override (kxmlgui);
+    // we need to create a new QSystemLocale to make Qt update the locale
+    qputenv("LANGUAGE", "ca:de_DE");
+    QSystemLocale newSystemLocale{};
+
+    // Initialize the testdata
+    QDir().mkpath(m_testConfigDir);
+    const QString file = m_testConfigDir + QLatin1String("/localized.test");
+    QFile::remove(file);
+    QFile f(file);
+    QVERIFY(f.open(QIODevice::WriteOnly));
+    QTextStream ts(&f);
+    ts << "[Test]\n";
+    ts << "foo=4\n";
+    ts << "foo[ca]=5\n";
+    ts << "foostring=primary\n";
+    ts << "foostring[ca]=nice\n";
+    ts << "foobool=primary\n";
+    ts << "foobool[ca]=true\n";
+    f.close();
+
+    // Load the testdata
+    QVERIFY(QFile::exists(file));
+    KConfig config(file);
+    QVERIFY(config.locale().startsWith("ca"_L1));
+
+    KConfigGroup cg2(&config, QStringLiteral("Test"));
+    QCOMPARE(cg2.readEntry("foo", 3), 5);
+    QCOMPARE(cg2.readEntry("foostring"), QStringLiteral("nice"));
+    QCOMPARE(cg2.readEntry("foobool", false), true);
+
+    // Clean up after the testcase
+    QFile::remove(file);
+}
+
 void KConfigTest::testDeleteWhenLocalized()
 {
     // Initialize the testdata
@@ -2025,7 +2085,7 @@ void KConfigTest::testXdgListEntry()
 }
 
 #include <QThreadPool>
-#include <qtconcurrentrun.h>
+#include <QtConcurrentRun>
 
 // To find multithreading bugs: valgrind --tool=helgrind --track-lockorders=no ./kconfigtest testThreads
 void KConfigTest::testThreads()
